@@ -5,6 +5,8 @@ from datetime import datetime
 import time
 import re
 import sqlite3
+from collections import defaultdict
+from threading import Lock
 import threading
 LOCAL = threading.local()
 
@@ -28,6 +30,7 @@ class EntryManager(object):
         self.index = index
         if not os.path.exists(self.index):
             self._create_table()
+        self.lock = defaultdict(Lock)
 
     # Thread-safe connection manager. Conections are stored in the 
     # ``threading.local`` object, so they can be safely reused in the
@@ -47,6 +50,7 @@ class EntryManager(object):
         self.conn.executescript("""
             CREATE TABLE flat (
                 id VARCHAR(255),
+                updated timestamp,
                 position CHAR(255),
                 leaf NUMERIC);
             CREATE INDEX position ON flat (position);
@@ -65,10 +69,7 @@ class EntryManager(object):
         curs = self.conn.execute("SELECT id FROM flat WHERE id=?", (id_,))
         if curs.fetchall():
             raise ConflictError('Conflict, the id "%s" already exists!' % id_)
-        updated = entry.setdefault('__updated__', datetime.utcnow())
-        if not isinstance(updated, datetime):
-            updated = datetime(
-                *(time.strptime(updated, '%Y-%m-%dT%H:%M:%SZ')[0:6]))
+        entry.setdefault('__updated__', datetime_to_iso(datetime.utcnow()))
 
         # Store entry.
         self.index_entry(entry)
@@ -81,7 +82,7 @@ class EntryManager(object):
         """, (key,))
         self.conn.commit()
 
-    def update(self, entry=None, **kwargs): 
+    def update(self, entry=None, condition=lambda old: True, **kwargs): 
         if entry is None:
             entry = kwargs
         else:
@@ -89,8 +90,13 @@ class EntryManager(object):
             entry.update(kwargs)
 
         id_ = entry['__id__']
-        self.delete(id_)
-        return self.create(entry)
+        with self.lock[id_]:
+            old = self.load_entry(id_)
+            if not condition(old):
+                raise ConflictError("Pre-condition failed!")
+            self.delete(id_)
+            new = self.create(entry)
+        return new
 
     def search(self, obj=None, size=None, offset=0, count=False, **kwargs):
         """
@@ -150,7 +156,7 @@ class EntryManager(object):
             query.append("(%s)" % " OR ".join(condition))
         if leaves:
             query.append("GROUP BY id HAVING COUNT(*)=%d" % leaves)
-        #query.append("ORDER BY updated DESC")
+        query.append("ORDER BY updated DESC")
         if size is not None:
             query.append("LIMIT %s" % size)
         if offset:
@@ -186,11 +192,11 @@ class EntryManager(object):
 
     def index_entry(self, entry):
         # Index entry.
-        indexes = [(entry['__id__'], k, v)
+        indexes = [(entry['__id__'], entry['__updated__'], k, v)
                 for (k, v) in flatten(entry) if k != '__id__']
         self.conn.executemany("""
-            INSERT INTO flat (id, position, leaf)
-            VALUES (?, ?, ?);
+            INSERT INTO flat (id, updated, position, leaf)
+            VALUES (?, ?, ?, ?);
         """, indexes)
         self.conn.commit()
 
@@ -223,10 +229,5 @@ def flatten(obj, keys=[]):
         for k, v in obj.items():
             for pair in flatten(v, keys + [escape(k)]):
                 yield pair
-    elif isinstance(obj, datetime):
-        yield key, datetime_to_iso(obj)
-    elif isinstance(obj, Operator):
-        obj.params = [datetime_to_iso(p) for p in obj.params]
-        yield key, obj
     else:
         yield key, obj
